/*                                                            -*- C -*-
 * Copyright (c) 2004 Takashi Nemoto 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <cstring>

#include "diskstat.h"

#ifdef ZAURUS
struct qpath_struct g_book_path [] =
  {
    {"/home/zaurus/dict/",			"HOME", 	MAIN0_READY},
    {"/home/zaurus/Documents/dict/", 		"DOCUMENT",	MAIN0_READY},
    {"/home/QtPalmtop/dict1/",			"QTPALMTOP1",	MAIN1_READY},
    {"/home/zaurus/Documents/dict1/", 		"DOCUMENT1",	MAIN1_READY},
    {"/home/QtPalmtop/dict2/",	 		"QTPALMTOP2",	MAIN2_READY},
    {"/home/zaurus/Documents/dict2/", 		"DOCUMENT2",	MAIN2_READY},
    {"/mnt/card/dict/", 			"SD",		SD_READY},
    {"/mnt/cf/dict/", 				"CF",		CF_READY},
    {"/mnt/net/dict/", 				"NET",		NET_READY},
    {NULL, 					"UNKNOWN",	0},
  };

struct qpath_struct g_appendix_path [] =
  {
    {"/home/zaurus/appendix/", 			"HOME", 	MAIN0_READY},
    {"/home/zaurus/Documents/dict/appendix/", 	"DOCUMENT",	MAIN0_READY},
    {"/home/QtPalmtop/dict1/appendix/",	 	"QTPALMTOP1",	MAIN1_READY},
    {"/home/zaurus/Documents/dict1/appendix/", 	"DOCUMENT1",	MAIN1_READY},
    {"/home/QtPalmtop/dict2/appendix/",		"QTPALMTOP2",	MAIN2_READY},
    {"/home/zaurus/Documents/dict2/appendix/", 	"DOCUMENT2",	MAIN2_READY},
    {"/mnt/card/appendix/", 			"SD",		SD_READY},
    {"/mnt/cf/appendix/", 			"CF",		CF_READY},
    {"/mnt/net/appendix/", 			"NET",		NET_READY},
    {"/usr/local/share/dict/app-pack/",		"APACK",        MAIN0_READY},
    {NULL, 					"UNKNOWN",	0},
  };
#else
struct qpath_struct g_book_path [] =
  {
    {NULL /* "HOME" */,				"HOME",		MAIN0_READY},
    {"/usr/share/dict/",			"MAIN", 	MAIN0_READY},
    {"/dict/", 					"ROOT",		MAIN0_READY},
    {NULL, 					"UNKNOWN",	0},
  };

struct qpath_struct g_appendix_path [] =
  {
    {NULL /* "HOME/appendix" */,		"HOME",		MAIN0_READY}, 
    {"/usr/share/dict/appendix/", 		"MAIN", 	MAIN0_READY},
    {"/dict/appendix/",			 	"ROOT",		MAIN0_READY},
    {"/usr/share/dict/app-pack/",		"APACK",        MAIN0_READY},
    {NULL, 					"UNKNOWN",	0},
  };
#endif

bool
DiskStat::checkDiskReady(bool enableSD, bool enableCF)
{
  struct stat st;
  int last_enabled=enabled;
  enabled = 0;
  for(qpath_struct* qps=g_book_path; qps->path!=NULL; qps++) {
    if (enabled_p(qps->id)) continue;
    if (qps->id==CF_READY && !enableCF) continue;
    if (qps->id==SD_READY && !enableSD) continue;
    if (stat(qps->path,&st)>=0) {
      enable(qps->id);
    }
  }
  if (enabled != last_enabled) return true;
  return false;
}

int DiskStat::memory(const char *path) 
{
  for(qpath_struct* qps=g_book_path; qps->path!=NULL; qps++) {
    if (strncmp(qps->path, path, strlen(qps->path))==0) {
      return qps->id;
    }
  }
  return 0;
}

const char *DiskStat::mem_text(const char *path) 
{
  for(qpath_struct* qps=g_book_path; qps->path!=NULL; qps++) {
    if (strncmp(qps->path, path, strlen(qps->path))==0) {
      return qps->name;
    }
  }
  return "???";
}

int DiskStat::enabled;

