/*                                                            -*- C -*-
 * Copyright (c) 2004 Takashi Nemoto 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include "zten.h"
#include "ztenconf.h"
#include "globalconfig.h"
#include <qlineedit.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qlabel.h>

#include <qlayout.h>
#include <qpushbutton.h>
#include <qhbox.h>

#define MAX_MAX_HITS 10000
#define MAX_INC_MAX_HITS 10000

QString GlobalConfig::KeyString(const KeyConfig& keyconf) {
  QString c,m;
  switch(keyconf.cursor) {
  case KEYCONF_UD:
    c=tr("Up/Down");
    break;
  case KEYCONF_LR:
    c=tr("Right/Left");
    break;
  case KEYCONF_LG:
    c=tr("Comma/Period");
    break;
  default:
    c=tr("???");
    break;
  }
  switch(keyconf.mod) {
  case KEYCONF_NONE:
    m=tr("None");
    break;
  case KEYCONF_SHIFT:
    m=tr("Shift");
    break;
  case KEYCONF_Fn:
    m=tr("Fn");
    break;
  case KEYCONF_CTRL:
    m=tr("Ctrl");
    break;
  default:
    m=tr("???");
    break;
  }
  return c+"  /"+m;
};

GlobalConfig::GlobalConfig (QWidget* parent, 
			    const char* name,
			    bool modal,
			    WFlags fl) 
  : GlobalConfigBase(parent, name, modal, fl) 
{
#ifndef QTOPIA
  QHBox* hbox=new QHBox(this);;
  QPushButton *acceptButton=new QPushButton(tr("Accept"),hbox);
  QPushButton *rejectButton=new QPushButton(tr("Reject"),hbox);
  hbox->setMaximumSize( QSize( 32000, 30 ) );
  GlobalConfigBaseLayout->addWidget(hbox);
  connect((QObject *)acceptButton, SIGNAL (clicked()), this, SLOT (accept ()));
  connect((QObject *)rejectButton, SIGNAL (clicked()), this, SLOT (reject ()));
#endif  

  ConfMaxHits->setText(QString("%1").arg(config->MaxHits));
  ConfIncMaxHits->setText(QString("%1").arg(config->IncrementalMaxHits));
  ConfIncMaxHitsParDic->setText(QString("%1").
				arg(config->IncrementalMaxHitsEach));
  IncrementalSearch->setChecked(config->IncrementalMode);
  ConfMultiMaxHits->setText(QString("%1").arg(config->MultiMaxHits));
  ConfKeywordMaxHits->setText(QString("%1").arg(config->KeywordMaxHits));
  BookTitleBeforeEntry->setChecked(config->BookNameBeforeEntry);
  //  CursorKeyForScroll->setChecked(config->CursorKeyForScroll);
  DicGroupName0->setText(config->GroupName[0]);
  DicGroupName1->setText(config->GroupName[1]);
  DicGroupName2->setText(config->GroupName[2]);
  DicGroupName3->setText(config->GroupName[3]);
  DicGroupName4->setText(config->GroupName[4]);
  DicGroupName5->setText(config->GroupName[5]);
  DicGroupName6->setText(config->GroupName[6]);
  DicGroupName7->setText(config->GroupName[7]);
  DicGroupName8->setText(config->GroupName[8]);
  DicGroupName9->setText(config->GroupName[9]);
  DicGroupNameS->setText(config->GroupName[DIC_GROUP_SINGLE]);
  DicGroupNameA->setText(config->GroupName[DIC_GROUP_ALL]);

  MaxContentLength->setText(QString("%1").arg(config->MaxContentLength));

  Stemming->setChecked(config->Stemming);
  UseSubSupFont->setChecked(config->UseSubSupFont);
  AutoImageDisplay->setChecked(config->AutoImageMode);

  ImageScaling->changeItem(QString("%1").arg(config->ImageScale)
			   +QString(" %"),0);
  ImageSizeLimitMode->setChecked(config->ImageSizeLimitMode);
  MaxImageWidth->setText(QString("%1").arg(config->MaxImageWidth));
  MaxImageHeight->setText(QString("%1").arg(config->MaxImageHeight));

  EnableQCopZtenv->setChecked(config->EnableQCopZtenv);
  EnableQCopZten->setChecked(config->EnableQCopZten);
  EnableQCopZdict->setChecked(config->EnableQCopZdict);

  UseProgressBar->setChecked(config->EnableProgressBar);
  UseScrollBar->setChecked(config->EnableScrollBar);

  MovieCommand->setText(config->MovieCommand);

  int keygroup=config->KeyConfigGroup;
  KeyConfigGroup->setCurrentItem(keygroup);

  KeyCommand->setCurrentItem(0);
  KeyCursor->setCurrentItem(config->keyconf[keygroup][0].cursor);
  KeyModifier->setCurrentItem(config->keyconf[keygroup][0].mod);

  KeyNextHit->setText(KeyString(config->keyconf[keygroup][KEYCONF_HIT]));
  KeyNextHist->setText(KeyString(config->keyconf[keygroup][KEYCONF_HIST]));
  KeyNextEntry->setText(KeyString(config->keyconf[keygroup][KEYCONF_ENTRY]));
  KeyScroll1Line->setText(KeyString(config->keyconf[keygroup][KEYCONF_SC1]));
  KeyScroll1Page->setText(KeyString(config->keyconf[keygroup][KEYCONF_SCP]));
}

GlobalConfig::~GlobalConfig() 
{
}

void GlobalConfig::accept ()
{
  config->MaxHits=ConfMaxHits->text().toInt();
  if (config->MaxHits<1 || config->MaxHits > MAX_MAX_HITS)
    config->MaxHits=200;
  config->IncrementalMaxHits=ConfIncMaxHits->text().toInt();
  if (config->IncrementalMaxHits<1 
      || config->IncrementalMaxHits >MAX_INC_MAX_HITS)
    config->IncrementalMaxHits=200;
  config->IncrementalMaxHitsEach=ConfIncMaxHitsParDic->text().toInt();
  if (config->IncrementalMaxHitsEach<1 || 
      config->IncrementalMaxHitsEach > config->IncrementalMaxHits)
    config->IncrementalMaxHits=config->IncrementalMaxHits;
  config->IncrementalMode=IncrementalSearch->isChecked();
  config->MultiMaxHits=ConfMultiMaxHits->text().toInt();
  if (config->MultiMaxHits<1 
      || config->MultiMaxHits >MAX_MAX_HITS)
    config->MultiMaxHits=200;
  config->KeywordMaxHits=ConfKeywordMaxHits->text().toInt();
  if (config->KeywordMaxHits<1 
      || config->KeywordMaxHits >MAX_MAX_HITS)
    config->KeywordMaxHits=200;
  config->MaxContentLength=MaxContentLength->text().toInt();
  config->BookNameBeforeEntry=BookTitleBeforeEntry->isChecked();
  //  config->CursorKeyForScroll=CursorKeyForScroll->isChecked();
  config->Stemming=Stemming->isChecked();
  config->AutoImageMode=AutoImageDisplay->isChecked();
  QString s=ImageScaling->currentText();
  if (s.length()>2) s=s.left(s.length()-2);
  config->ImageScale=s.toInt();
  config->ImageSizeLimitMode=ImageSizeLimitMode->isChecked();
  config->MaxImageWidth=MaxImageWidth->text().toInt();
  config->MaxImageHeight=MaxImageHeight->text().toInt();

  config->MovieCommand=MovieCommand->text();
  config->UseSubSupFont=UseSubSupFont->isChecked();

  config->EnableQCopZtenv=EnableQCopZtenv->isChecked();
  config->EnableQCopZten =EnableQCopZten->isChecked();
  config->EnableQCopZdict=EnableQCopZdict->isChecked();

  config->GroupName[0]=DicGroupName0->text();
  config->GroupName[1]=DicGroupName1->text();
  config->GroupName[2]=DicGroupName2->text();
  config->GroupName[3]=DicGroupName3->text();
  config->GroupName[4]=DicGroupName4->text();
  config->GroupName[5]=DicGroupName5->text();
  config->GroupName[6]=DicGroupName6->text();
  config->GroupName[7]=DicGroupName7->text();
  config->GroupName[8]=DicGroupName8->text();
  config->GroupName[9]=DicGroupName9->text();
  config->GroupName[DIC_GROUP_SINGLE]=DicGroupNameS->text();
  config->GroupName[DIC_GROUP_ALL]=DicGroupNameA->text();

  config->EnableProgressBar=UseProgressBar->isChecked();
  config->EnableScrollBar=UseScrollBar->isChecked();

  config->KeyConfigGroup=KeyConfigGroup->currentItem();

  GlobalConfigBase::accept();
}

