/*                                                            -*- C -*-
 * Copyright (c) 2003-2004 Teranishi & Nemoto 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */


#include <qapplication.h>
#include <qclipboard.h>
#include <qlineedit.h>
#include <qcombobox.h>
#include <qlistbox.h>
#include <qlistview.h>
#include <qlayout.h>
#include <qarray.h>
#include <qstrlist.h>
#include <qtabwidget.h>
#include <qfont.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qstring.h>
#include <qtextcodec.h>
#include <qimage.h>
#include <qtimer.h>
#include <qtoolbar.h>
#include <qpopupmenu.h>
#include <eb/eb.h>

#include <dlfcn.h>
#include <sys/stat.h>

#include "qbookentry.h"
#include "qbookbrowser.h"
#include "qhistorylineedit.h"
#include "ztenwindow.h"
#include "ztenconf.h"
#include "globalconfig.h"
#include "ztendump.h"
#include "qbookitem.h"
#include "ztenfontmenu.h"
#include "ztensound.h"

#include "zpixmap.h"

#include "zten.h"
#include "ztenext.h"
#include "ztenbooklist.h"
#include "ztenmultisearch.h"
#include "ztencandidate.h"
#include "qlistboxrichtext.h"
#include "diskstat.h"

extern "C" {
  // #include <libkakasi.h>
  void* z_kakasi_dlhandle=NULL;
  int (*z_kakasi_getopt_argv)(int argc, char **argv) = NULL;
  char* (*z_kakasi_do)(char *str) = NULL;
  int (*z_kakasi_close_kanwadict)(void) = NULL;
  int (*z_kakasi_free)(char *p) = NULL;
  void (*z_kakasi_itaijiknj)(int*,int*) = NULL;

  extern const char* VERSION;
}

#ifdef QTOPIA
  #include <qpe/config.h>
  #include <qpe/qpeapplication.h>
  #include <qpe/qcopenvelope_qws.h>
  extern QPEApplication *app;
#else
  #include <qapplication.h>
  extern QApplication *app;
#endif

#include <qprogressbar.h>

ZtenConf* config;

QBookHitItem::QBookHitItem (const QString &label, QBookEntry *be,
			    int hit_pos)
  : QListBoxRichText (label, label)
{
  this->be = be;
  this->hit_pos = hit_pos;
}

QBookHitItem::~QBookHitItem ()
{
}

bool
QBookMultiSearch::operator <(const QBookMultiSearch& qms) { 
  return be->priority[QBookEntry::group]<qms.be->priority[QBookEntry::group]; 
};

bool 
QBookMultiSearch::operator ==(const QBookMultiSearch& qms) { 
  return be->priority[QBookEntry::group]==qms.be->priority[QBookEntry::group]; 
};

void Zten::setFocus(int fval) {
  focus=fval;
  if (config->clipSearchMode && !MainWindow->menuBarBase->isVisible()) {
    if (current_content_mode==MaximizeEntryList) {
      EntryList->setFocus();
    } else {
      ContentBrowser->setFocus();
    } 
  } else {
    LineEdit->setFocus(); // Always "Line Edit Mode"
  }
}

void
Zten::entryListEscape ()
{
  contentMode (MaximizeEntryList, false);
  setFocus(FocusLineEdit);
}

void
Zten::entryListUp ()
{
  QListBoxItem *item;
  if (EntryList->count()==0) return;
  contentMode (AddEntryList, false);
  int i = EntryList->currentItem ();
  if (i >= 0) {
    if ((item = EntryList->item(i)->prev())) {
      setFocus(FocusEntryList);
      EntryList->setCurrentItem (item);
    } else {
      EntryList->setSelected(0,false);
      entryListSelected=false;
      LineEdit->lineEdit()
	->setSelection(LineEdit->lineEdit()->text().length(),0);
      LineEdit->lineEdit()->setCursorPosition
	(LineEdit->lineEdit()->text().length());
      contentMode (MaximizeEntryList, false);
      setFocus(FocusLineEdit);
    }
  } else {
    contentMode (MaximizeEntryList, false);
    setFocus(FocusLineEdit);
  }
}

void
Zten::entryListDown ()
{
  QListBoxItem *item;
  if (EntryList->count()==0) return;

  contentMode (AddEntryList, false);

  int cur= EntryList->currentItem ();
  if (focus == FocusLineEdit) {
    if (cur<0 || !EntryList->isSelected(cur) || entryListSelected==false) {
      EntryList->setSelectionMode(QListBox::Single);
      EntryList->setSelected(0,true);
      entryListSelected=true;
      setFocus(FocusEntryList);
      return;
    }
  }
  if (cur >= 0) {
    if ((item = EntryList->item(cur)->next())) {
      setFocus(FocusEntryList);
      EntryList->setCurrentItem (item); 
    }
  }  
}

void
Zten::contentLineDown ()
{
  if (displaying)
    {
      ContentBrowser->scrollBy (0, config->FontSize);
    }
  else
    {
      contentMode (DisplayBoth, false);
    }
}

void
Zten::contentLineUp ()
{
  if (displaying) {
    ContentBrowser->scrollBy (0, -config->FontSize);
  } else {
    contentMode (DisplayBoth, false);
  }  
}

void
Zten::contentPageUp ()
{
  if (displaying) {
    if (ContentBrowser->contentsY () == 0) {
    } else {
      ContentBrowser->scrollBy (0, ContentBrowser->height () * -9 / 10);
    }
  } else {
    EntryList->scrollBy (0, EntryList->height () * -9 / 10);
    unsigned int i,b=0,e=0;
    int c=0;
    for(i=0;i<EntryList->count();i++){
      if (EntryList->itemVisible(i)) {
	b=i;
	break;
      }
    }
    for(;i<EntryList->count();i++){
      if (!EntryList->itemVisible(i)) {
	break;
      }
    }
    e=i;
    if (e>b) c=(b+e)/2; else c=0;
    if (c>int(EntryList->count())-1) c=EntryList->count()-1;
    if (c<0) c=0;
    EntryList->setCurrentItem(c);
  }
}

void
Zten::contentPageDown ()
{
  if (displaying) {
    ContentBrowser->scrollBy (0, ContentBrowser->height () * 9 / 10);
  } else {
    EntryList->scrollBy (0, EntryList->height () * 9 / 10);
    int b=0,e=0,c=0;
    unsigned int i;
    for(i=0;i<EntryList->count();i++){
      if (EntryList->itemVisible(i)) {
	b=i;
	break;
      }
    }
    for(;i<EntryList->count();i++){
      if (!EntryList->itemVisible(i)) {
	break;
      }
    }
    e=i;
    if (e>b) c=(b+e)/2; else c=EntryList->count()-1;
    if (c>int(EntryList->count())-1) c=EntryList->count()-1;
    if (c<0) c=0;
    EntryList->setCurrentItem(c);
  }
}

void
Zten::toggleDirection ()
{
  if (BookContentLayout->direction () == QBoxLayout::LeftToRight)
    {
      EntryList->setMinimumSize(QSize(ContentBrowser->width (),
				      ContentBrowser->height () / 3));
      BookContentLayout->setDirection (QBoxLayout::TopToBottom);
    }
  else
    {
      EntryList->setMinimumSize(QSize(ContentBrowser->width () / 3,
				      ContentBrowser->height ()));      
      BookContentLayout->setDirection (QBoxLayout::LeftToRight);
    }
}

void
Zten::resizeEvent (QResizeEvent* e)
{
  QSize s = e->size ();
  if (s.width () > s.height ())
    {
      BookContentLayout->setDirection (QBoxLayout::LeftToRight);
    }
  else
    {
      BookContentLayout->setDirection (QBoxLayout::TopToBottom);      
    }
  // FIXME: hard coding.
  EntryList->setMinimumSize(QSize(s.width () / 3,
				  s.height () / 4));
  ZtenBase::resizeEvent (e);
}

void
Zten::contentMode (content_mode mode, bool redrawContent)
{
  if (config->EnableScrollBar) {
    ContentBrowser->setVScrollBarMode( QTextBrowser::Auto);
    ContentBrowser->setHScrollBarMode( QTextBrowser::Auto);
    EntryList->setVScrollBarMode( QScrollView::Auto );
  } else {
    ContentBrowser->setVScrollBarMode( QTextBrowser::AlwaysOff );
    ContentBrowser->setHScrollBarMode( QTextBrowser::AlwaysOff );
    EntryList->setVScrollBarMode( QScrollView::AlwaysOff );
  }

  if (mode == AddContent) {
    if (current_content_mode == MaximizeEntryList) {
      mode = DisplayBoth;
    } else {
      return;
    }
  } else if(mode == AddEntryList) {
    if (current_content_mode == MaximizeContent) {
      mode = DisplayBoth;
    } else {
      return;
    }
  }
  if (mode == LargerContent) {
    if (current_content_mode == MaximizeEntryList) {
      mode = DisplayBoth;
    } else {
      mode = MaximizeContent;
    }
  } else if(mode == LargerEntryList) {
    if (current_content_mode == MaximizeContent) {
      mode = DisplayBoth;
    } else {
      mode = MaximizeEntryList;
    }
  } else if(mode == ToggleEntryList) {
    if (current_content_mode == MaximizeContent) {
      mode = MaximizeEntryList;
    } else if (current_content_mode == MaximizeEntryList) {
      mode = DisplayBoth;
    } else {
      mode = MaximizeContent; 
    }
  }
  current_content_mode=mode;
  if (mode == MaximizeEntryList) {
    setFocus(FocusEntryList);
    EntryList->show ();
    ContentBrowser->hide ();
    displaying = false;
  } else if (mode == MaximizeContent) {
    setFocus(FocusContentBrowser);
    ContentBrowser->show ();
    EntryList->hide ();
    if (history_current >= 0 && redrawContent) goToHistory(history_current);
    displaying = true;
  } else {
    setFocus(FocusEntryList);
    ContentBrowser->show ();
    EntryList->show ();
    if (history_current >= 0 && redrawContent) goToHistory(history_current);
    displaying = true;
  }
  ZtenBaseLayout->activate();
}

void
Zten::linkJump (int pos)
{
  if (current_book==NULL) return;
  EB_Position p = current_book->getLinkPosition (pos);
  switch(p.page) {
  case LINK_SOUND:
    current_book->playSound(p.offset);
    break;
  case LINK_MOVIE:
    current_book->playMovie(p.offset);
    break;
  case LINK_IMAGE:
    current_book->showImage(p.offset);
    break;
#if 0
  case LINK_XASOUND:
    current_book->playXASound(p.offset);
    break;
#endif
  default:
    if (p.page > 0) goTo (p); 
  }
}

void Zten::linkJump() {
  QString txt=JumpEdit->text();
  JumpEdit->setText("");
  LineEdit->setFocus();
  if (txt.length()<1 || txt.length()>2) return;
  linkJump(-txt.toInt(0,36));
}

bool Zten::eventFilterCommon(QObject * /* o */, QEvent *e){
  if (e->type () == QEvent::KeyPress) {
    QKeyEvent *k = (QKeyEvent *)e;
    switch (k->key ()) {
    case Qt::Key_End:
      forward(); // history
      return TRUE;

    case Qt::Key_Home:
      back(); // history
      return TRUE;

    case Qt::Key_BackSpace:
      if ((k->state() & Qt::ControlButton) || (k->state() & Qt::ShiftButton)) {
	toggleDirection ();
	return TRUE;
	//	contentMode(ToggleEntryList, false);
	//	return TRUE;
      }
      break;
    case Qt::Key_F33:  // Key "Ok"
      if (config->clipSearchMode) {
	config->clipSearchMode=false;
	MainWindow->drawToolBar();
	return TRUE;
      }
    }

    if (k->state() & Qt::ControlButton) {
      switch (k->key ()) {

      case Qt::Key_1:
      case Qt::Key_2:
      case Qt::Key_3:
      case Qt::Key_4:
      case Qt::Key_5:
      case Qt::Key_6:
      case Qt::Key_7:
      case Qt::Key_8:
      case Qt::Key_9:
      case Qt::Key_0:
      case Qt::Key_A:
      case Qt::Key_B:
	//      case Qt::Key_C:
      case Qt::Key_D:
      case Qt::Key_E:
      case Qt::Key_F:
      case Qt::Key_G:
      case Qt::Key_H:
      case Qt::Key_I:
      case Qt::Key_J:
      case Qt::Key_K:
      case Qt::Key_L:
      case Qt::Key_M:
      case Qt::Key_N:
      case Qt::Key_O:
      case Qt::Key_P:
      case Qt::Key_Q:
      case Qt::Key_R:
      case Qt::Key_S:
      case Qt::Key_T:
      case Qt::Key_U:
	//      case Qt::Key_V:
      case Qt::Key_W:
	//      case Qt::Key_X:
      case Qt::Key_Y:
      case Qt::Key_Z:
	linkJump (k->key ());
	return TRUE;
      case Qt::Key_C:
	copy();
	return TRUE;
      } 
    } 
    return false;
  }
  return false;
}

bool
Zten::eventFilter (QObject *o, QEvent *e)
{
  /*
  if (config->clipSearchMode && 
      e->type () == QEvent::KeyPress) {
    QKeyEvent *k = (QKeyEvent *)e;
    switch (k->key ()) {
    case Qt::Key_F33:  // Key "Ok"
	config->clipSearchMode=false;
	MainWindow->drawToolBar();
	return TRUE;
    }
  }
  */

  if (e->type () == QEvent::KeyPress) {
    QKeyEvent *k = (QKeyEvent *)e;
    switch (k->key ()) {
    case Qt::Key_Escape:
      // fprintf(stderr,"Escape\n");
      //      MainWindow->setWFlags(WType_TopLevel);
      //	MainWindow->reparent(0,WType_TopLevel,QPoint(0,0));
      if (config->clipSearchMode) {
	config->clipSearchMode=false;
	MainWindow->drawToolBar();
	return TRUE;
      } else if (MainWindow->testWFlags(WStyle_NoBorderEx)) {
	MainWindow->reparent(0,WType_TopLevel,QPoint(0,0));
	//	app->quit();
	return TRUE;
      }
    }
  }


  if (JumpEdit->hasFocus()) {
    return eventFilterJump(o,e);
  }
  if (focus==FocusContentBrowser) {
    if (eventFilterMaximizedContent(o,e)) return TRUE;
  } else {
    if (eventFilterNormal(o,e)) return TRUE;
  }
  if (eventFilterCommon(o,e)) return TRUE;
  return QWidget::eventFilter (o, e);
}

bool Zten::eventFilterJump(QObject *o, QEvent *e) {
  if (e->type () == QEvent::KeyPress) {
    QKeyEvent *k = (QKeyEvent *)e;
    switch(k->key()) {
    case Key_Left:
      LineEdit->setFocus();
      return TRUE;
    }
  }  
  return QWidget::eventFilter(o,e);
}

bool Zten::eventFilterMaximizedContent (QObject * /* o */, QEvent *e)
{
  if (e->type () == QEvent::KeyPress) {
    QKeyEvent *k = (QKeyEvent *)e;

    switch(k->key()) {

    case Qt::Key_Next:
      contentPageDown ();
      return TRUE;

    case Qt::Key_Prior:
      contentPageUp ();
      return TRUE;

    case Qt::Key_Down:
      if (k->state() & Qt::ShiftButton) {
	down();
	return TRUE;
      } else {
	contentLineDown();
	return TRUE;
      }
      
    case Qt::Key_Up:
      if (k->state() & Qt::ShiftButton) {
	up();
	return TRUE;
      } else {
	contentLineUp();
	return TRUE;
      }

    case Qt::Key_Left:
      
      contentMode(LargerEntryList,false);
      //      setFocus(FocusEntryList);
      return TRUE;

    case Qt::Key_Right:
      if (JumpEdit->isVisible()) JumpEdit->setFocus();
      return TRUE;

    case Qt::Key_Return:
      contentMode(MaximizeEntryList,false);
      //      setFocus(FocusEntryList);
      return TRUE;

    case Key_X:
      if (k->state() & Qt::ControlButton) {
	cut();
	return TRUE;
      }
      break;
    case Key_C:
      if (k->state() & Qt::ControlButton) {
	copy();
	return TRUE;
      }
      break;
    case Key_V:
      if (k->state() & Qt::ControlButton) {
	paste();
	return TRUE;
      }
    }
  }
  return false;
}


bool
Zten::eventFilterNormal (QObject * /* o */, QEvent *e)
{
  if (e->type () == QEvent::KeyPress) {
    QKeyEvent *k = (QKeyEvent *)e;
    //    qDebug( "Key press %d", k->key() );
    int key=k->key();
    switch (key) {
    case Qt::Key_Next:
      contentPageDown ();
      return TRUE;
    case Qt::Key_Prior:
      contentPageUp ();
      return TRUE;
    case Qt::Key_Down:
      if (k->state() & Qt::ShiftButton) {
	down();
      } else {
	entryListDown ();
      }
      return TRUE;
    case Qt::Key_Up:
      if (k->state() & Qt::ShiftButton) {
	up();
      } else {
	entryListUp ();
      }
      return TRUE;
    case Qt::Key_Left:
      if (current_content_mode==MaximizeEntryList) {
	if (config->clipSearchMode) {
	  config->clipSearchMode=false;
	  MainWindow->drawToolBar();
	}
	EntryList->clear();
	setFocus(FocusLineEdit);
	break;
      } else {
	contentMode(LargerEntryList,false);
	return TRUE;
      }

    case Qt::Key_Right:
      if (LineEdit->lineEdit()->cursorPosition() == 
	  int(LineEdit->lineEdit()->text().length())) {
	execSearch2();
	return TRUE;
      }
      break;

    case Qt::Key_Tab:
      break;

      //    default:
      //      if (o==EntryList) {
	// setFocus(FocusLineEdit);
	//	QWidget::eventFilter (LineEdit->lineEdit(), e);
	//	QWidget::eventFilter (LineEdit, e);
      //      }
    }
  }
  return FALSE;
}

void Zten::showBlank() {
  ContentBrowser->setText(tr("Ztenv version ")+QString(VERSION)+"\n"+
			  tr("Blank Text"));
}

void Zten::showCopyright() {
  if (QBookEntry::group==DIC_GROUP_SINGLE) {
    for (QBookEntry* be = books.first ();  be != NULL; be = books.next ()) {
      if (be->priority[QBookEntry::group]>0) {
	EB_Position pos;
	if (eb_have_copyright(be->getBook()) &&
	    EB_SUCCESS==eb_copyright(be->getBook(), &pos)) {
	  current_book = be;
	  ContentBrowser->setText("");
	  contentMode(MaximizeContent, false);
	  goToNoHistory (pos);
	  return;
	}
      }
    }
  } 
}

void
Zten::initializeGUI() {
  ztenBookList = new ZtenBookList(this);
  DictionaryList = ztenBookList->DictionaryList;

  EntryList->setHScrollBarMode( QScrollView::AlwaysOff );
  ContentBrowser->setHScrollBarMode( QTextBrowser::AlwaysOff );

  BookContentLayout->setStretchFactor(EntryList, 1);
  BookContentLayout->setStretchFactor(ContentBrowser, 2);

  contentMode(MaximizeContent,false);
  showBlank();
}

Zten::Zten(QWidget* parent, const char* name, WFlags fl)
  : ZtenBase(parent, name, fl)
{
  eb_initialize_library();
  focus = FocusLineEdit;

  waitCursorMode=false;

  QBookEntry::back_color = colorGroup().highlight ().rgb ();
  QBookEntry::fore_color = colorGroup().highlightedText ().rgb ();
  
  z_kakasi_dlhandle=dlopen("libkakasi.so",RTLD_LAZY);
  if (z_kakasi_dlhandle!=NULL) {
    z_kakasi_getopt_argv=
      (int(*)(int,char**))dlsym(z_kakasi_dlhandle,"kakasi_getopt_argv");
    z_kakasi_do=
      (char* (*)(char *)) dlsym(z_kakasi_dlhandle,"kakasi_do");
    z_kakasi_close_kanwadict=
      (int (*)(void))dlsym(z_kakasi_dlhandle,"kakasi_close_kanwadict");
    z_kakasi_free=
      (int (*)(char *))dlsym(z_kakasi_dlhandle,"kakasi_free");
    // internal function but....
    z_kakasi_itaijiknj=
      (void (*)(int*,int*))dlsym(z_kakasi_dlhandle,"itaijiknj");
  }
  char* kakasi_args [] = {NULL, "-JH"};
  if (z_kakasi_getopt_argv!=NULL) {
    char cwd[1024];
    struct stat fst;
    if (lstat("/opt/QtPalmtop/etc/kanwadict",&fst)==0) {
      // for SL-C3000 internal kakasi library
      getcwd(cwd,1024);
      int a=0,b=0;
      chdir("/opt/QtPalmtop/etc");
      if (z_kakasi_getopt_argv!=NULL && z_kakasi_itaijiknj!=NULL) {
	z_kakasi_getopt_argv (2, kakasi_args);
	z_kakasi_itaijiknj(&a,&b);
      }
      chdir(cwd);
    } else {
      z_kakasi_getopt_argv (2, kakasi_args);
    }
  }

  DelButton = ((ZtenWindow*)parent)->DelButton;
  LineEdit = ((ZtenWindow*)parent)->LineEdit;
  JumpEdit =  ((ZtenWindow*)parent)->JumpEdit;
  //  LookupButton = ((ZtenWindow*)parent)->LookupButton;
  MainWindow = (ZtenWindow*) parent;

  config = new ZtenConf("ztenv");
  displaying = TRUE;

  ms_list.setAutoDelete (TRUE);

  current_book = NULL;
  history.resize (0);
  history_book.resize (0);
  history_current = -1;

  /*
  connect ((QObject *)LookupButton, SIGNAL (clicked()),
	   this, SLOT (activateAndSearch()));
  */
  connect ((QObject *)DelButton, SIGNAL (clicked()),
	   this, SLOT (clearLine()));
  connect (LineEdit->lineEdit(), SIGNAL (returnPressed ()), this, 
	   SLOT (execSearch()));  
  connect (JumpEdit, SIGNAL (returnPressed ()), this, 
	   SLOT (linkJump()));  
  connect (LineEdit, SIGNAL (downPressed ()), this, SLOT (entryListDown()));
  connect (LineEdit, SIGNAL (textChanged (const QString &)),
	   this, SLOT (searchIncremental (const QString &)));
  connect (EntryList, SIGNAL (currentChanged (QListBoxItem *)), 
	   this, SLOT (itemSelected (QListBoxItem *)));
  connect (EntryList, SIGNAL (clicked (QListBoxItem *)), 
    	   this, SLOT (itemPressed (QListBoxItem *)));

  MenuSearch = new QPopupMenu(this);
  connect (MenuSearch, SIGNAL (activated (int)),
  	   this, SLOT (menuSelected (int)));  

  MultiSearch = new QPopupMenu(this);
  connect (MultiSearch, SIGNAL (activated (int)), 
	   this, SLOT (multiSelected(int)));
			      
  connect (ContentBrowser, SIGNAL (goTo (EB_Position)),
	   this, SLOT (goTo (EB_Position)));
  connect (ContentBrowser, SIGNAL (playSound (int)),
	   this, SLOT (playSound (int)));
  connect (ContentBrowser, SIGNAL (playMovie (int)),
	   this, SLOT (playMovie (int)));
  connect (ContentBrowser, SIGNAL (showImage (int)),
	   this, SLOT (showImage (int)));
  connect (ContentBrowser, SIGNAL (showImageGroup (int)),
	   this, SLOT (showImageGroup (int)));

  //  opened = FALSE;  

  LineEdit->lineEdit()->installEventFilter (this);
  ContentBrowser->installEventFilter (this);
  EntryList->installEventFilter (this);
  JumpEdit->installEventFilter(this);
  
  contentMode(MaximizeEntryList,false);

  ContentBrowser->setFocusPolicy(QWidget::NoFocus);
  EntryList->setFocusPolicy(QWidget::NoFocus);

  initializeGUI();

  emit noHistory ();

}

Zten::~Zten()
{
  config->writeConfig (this);
  delete config;
  delete cg;
  eb_finalize_library();
}

void
Zten::setImageScale (int scale)
{
  config->ImageScale=scale;
  if (history_current >= 0)
    goToHistory(history_current);
}
  
void
Zten::setCurrentFont (const QFont& font)
{
  config->FontSize=font.pixelSize();
  config->FontFamily=font.family();
  DictionaryList ->setFont (font);
  EntryList->setFont (font);
  ContentBrowser->setFont (font);
  QBookEntry::setFont(font);
  if (history_current >= 0)
    goToHistory (history_current);
}

void
Zten::clearBooks ()
{
  DictionaryList->clear ();
}

void
Zten::addBook(QBookEntry *item)
{
  for(int i=0;i<EB_MAX_MULTI_SEARCHES;i++){
    item->multi_search_enabled[i]=false;
  }
  item->menu_enabled=false;
  books.append(item);
}

void
Zten::setExtension (QBookEntry *be)
{
  ZtenExtension ext (be, this, NULL, true);
#ifdef ZAURUS
  ext.showMaximized ();
#else
  ext.show();
#endif
  ext.exec ();
}

void Zten::updateDicList(bool sort) {
  if (sort) books.sort();

  QString s;
  DictionaryList->clear();
  int count=0;
  for(QBookEntry *be = books.first(); books.current(); be = books.next()) {
    QBookItem *bi = new QBookItem(DictionaryList);
    bi->setText(0, be->getTitle());
    bi->setText(1, DiskStat::mem_text(be->absolute_path));
    for(int i=0; i<MAX_DIC_GROUPS+2; i++){
      if (be->priority[i]==0) {
	s=QString("ZZZ");
      } else {
	s.sprintf("%03d",be->priority[i]);
      }
      bi->setText(2+i, s);
    }
    bi->setPath(be->absolute_path);
    bi->setSubBookPos(be->sub_book_pos);
    bi->setID(count++);
    bi->setColor(be->alt_title_color);
  }
  if (sort) DictionaryList->setSorting(2+QBookEntry::group);
}

void Zten::dicList ()
{
  updateDicList();
#ifdef ZAURUS
  ztenBookList->showMaximized ();
#else
  ztenBookList->show ();
#endif
  if (ztenBookList->exec () == QDialog::Accepted) {
    config->writeConfig(this);
  }
  openBooks(false,NULL);
  findMenues();
  findMultiSearches ();
  MainWindow->setupDictionaryGroupMenu();
}

void
Zten::itemSelected (QListBoxItem *item)
{
  if (item)
    {
      ContentBrowser->setText(tr("Wait !"));
      //      app->processEvents();
      EntryList->setSelectionMode(QListBox::Single);
      QBookHitItem *bi = (QBookHitItem *)item;
      QBookEntry *be = bi->getBookEntry();
      current_book = be;
      if (be) {
	goTo(be->getHitPosition(bi->getHitPos()));
      }
    }
}

void
Zten::itemPressed (QListBoxItem *item)
{
  contentMode(LargerContent,false);
  itemSelected(item);

  LineEdit->lineEdit()->selectAll();
  if (current_content_mode == DisplayBoth) {
    setFocus(FocusEntryList);
  } else {
    setFocus(FocusContentBrowser);
  }
}

void
Zten::lookup (QString key)
{
  LineEdit->setEditText (key);
  search ();
}

void
Zten::setIncremental (bool on)
{
  config->IncrementalMode = on;
  UpdateConfig();
}

void
Zten::setAutoContent (bool on)
{
  config->AutoContentMode = on;
  UpdateConfig();
}

void
Zten::setClipSearch (bool on)
{
  config->EnableClipSearch = on;
  UpdateConfig();
}

void Zten::UpdateConfig() {
  config->writeConfig(this);
}

void
Zten::searchIncremental (const QString & word)
{
  int count=0;
  QMimeSourceFactory::setDefaultFactory(new QMimeSourceFactory());

  contentMode(MaximizeEntryList, false);
  EntryList->clear();

  if (config->IncrementalMode) {
    if (word.length () >= 1) {
      QBookEntry *be;
      int matches;
      DiskStat::checkDiskReady(config->EnableSD, config->EnableCF);
      for (be = books.first (); be != NULL; be = books.next ()) {
	if (be->priority[0]==0 || !be->isReady()) continue;
	if (count >= config->IncrementalMaxHits) break;
	matches = be->search (word, FORWARD_MATCH);
	int count2=0;

	for (int i = 0 ; 
	     i < matches && count2 < config->IncrementalMaxHitsEach; i ++) {
	  QString heading = be->getHeading (i);
	  if (!heading.isNull()) {
	    if (config->BookNameBeforeEntry) {
	      EntryList->insertItem (new QBookHitItem
				     ("<font color=\"" +
				      be->getAltTitleColor () +
				      "\">" +
				      be->getAltTitle () + 
				      "</font> " +
				      heading, 
				      be, i), -1);
	    } else {
	      EntryList->insertItem (new QBookHitItem
				     (heading + " <font color=\"" +
				      be->getAltTitleColor () +
				      "\">" +
				      be->getAltTitle () + 
				      "</font> ",
				      be, i), -1);
	      /*
	      EntryList->insertItem (new QBookHitItem
				     (heading + " [" +
				      be->getTitle () + "]",
				      be, i), -1);
	      */
	    }
	    count++;
	    count2++;
	  }
	}
      }
    } else {
      if (QBookEntry::group==DIC_GROUP_SINGLE) {
	showCopyright(); 
      } else {
	ContentBrowser->setText ("");
      }
    }
  }
  setFocus(FocusLineEdit);
}

void
Zten::activateAndSearch()
{
  LineEdit->activate ();
  search ();
}

void
Zten::clearLine ()
{
  setFocus(FocusLineEdit);
  LineEdit->setEditText ("");
}

void Zten::execSearch () {
  StopSound();
  if (focus==FocusLineEdit) {
    search();
  } else if (focus==FocusEntryList) {
    contentMode(MaximizeContent,false);
    itemPressed(EntryList->item(EntryList->currentItem()));
  }
}

void Zten::execSearch2 () {
  StopSound();
  if (focus==FocusLineEdit) {
    search();
  } else if (focus==FocusEntryList) {
    itemPressed(EntryList->item(EntryList->currentItem()));
  }
}


void Zten::setWaitCursor() {
  if (!waitCursorMode) {
    /*
    {
      QCopEnvelope e("QPE/System","busy()");
    }
    //    app->setOverrideCursor(WaitCursor);
    */
    //    sl_dlg_wait=new SlDlgWait(MainWindow,tr("Please Wait !!"),TRUE,TRUE);
    /*
    if (config->EnableWaitDialog) {
      SlDlgWait sl_dlg_wait(MainWindow,tr("Please Wait !!"));
      sl_dlg_wait.exec();
    }
    */
    waitCursorMode = true;
  }
}

void Zten::resetWaitCursor() {
  if (waitCursorMode) {
    /*
    {
      QCopEnvelope e("QPE/System","notBusy()");
    }
    */
    //    app->restoreOverrideCursor();
    waitCursorMode = false;
  }
}

void Zten::idle_in(void) {
  resetWaitCursor();
}

void
Zten::search ()
{
  QBookEntry *be;
  int match_mode = config->CurrentMatch;
  QString word = LineEdit->currentText();
  QMimeSourceFactory::setDefaultFactory(new QMimeSourceFactory());

 retry:
  int total_matches = 0;
  int dicCount=0;
  int matches;
  QBookEntry *first_match = NULL;

  setWaitCursor();

  EntryList->clear();
  contentMode(MaximizeEntryList, false);

  if (word.length() == 0) {
    if (QBookEntry::group==DIC_GROUP_SINGLE) {
      showCopyright(); 
    } else {
      ContentBrowser->setText ("");
    }
    return;
  }
  for (be = books.first (); be != NULL; be = books.next ())  {
    if (be->priority[QBookEntry::group]<=0) continue;
    if (config->clipSearchMode && 
	((dicCount>=config->ClipSearchMaxDicts) ||
	 (total_matches>=config->ClipSearchMaxDicts))) break;
    dicCount++;
    matches = be->search (word, match_mode);
    if (matches<0) continue;
    int count=0;
    for (int i = 0 ; i < matches; i ++) {
      if (first_match == NULL) {
	first_match = be;
      }
      QString heading = be->getHeading (i);
      if (!heading.isNull()) {
	if (config->BookNameBeforeEntry) {
	  EntryList->insertItem (new QBookHitItem
				 ("<font color=\"" +
				  be->getAltTitleColor () +
				  "\">" +
				  be->getAltTitle () + 
				  "</font> " +
				  heading, 
				  be, i), -1);
	} else {
	      EntryList->insertItem (new QBookHitItem
				     (heading + " <font color=\"" +
				      be->getAltTitleColor () +
				      "\">" +
				      be->getAltTitle () + 
				      "</font> ",
				      be, i), -1);
	      /*
		EntryList->insertItem (new QBookHitItem
		(heading + " [" +
		be->getTitle () + "]",
		be, i), -1);
	      */
	}
	count++;
      }
      if (config->clipSearchMode && 
	  count>=config->ClipSearchMaxHitsEach) break;
    }
    total_matches+=count;
  }
  resetWaitCursor();

  if (dicCount == 0) {
    contentMode(MaximizeContent, false);
    ContentBrowser->setText(tr("No dictionary"));
  } else if (total_matches == 0) {
    if (config->Stemming) {
      // Very simple stemming... 
      if (match_mode==EXACT_MATCH) {
	match_mode = FORWARD_MATCH;
	goto retry;
      } else {
	if (word.length()>1) {
	  word.remove(word.length()-1,1);
	  LineEdit->setEditText(word+"*");
	  goto retry;
	}
      }
    }
    contentMode(MaximizeContent, false);
    ContentBrowser->setText(tr("No matches"));
  } else {
    EntryList->setSelectionMode(QListBox::Single);
    EntryList->setSelected(0, true);
    setFocus(FocusEntryList);
    
    fprintf(stderr,"%d %d %d\n", total_matches, config->AutoContentMode, config->clipSearchMode);
    if (/*total_matches == 1 ||*/ 
	(total_matches>0 && 
	 (config->AutoContentMode
	  || config->clipSearchMode))) {
      fprintf(stderr,"Hit!\n");
      itemPressed(EntryList->item(0));
    }
  }
}

void 
Zten::addHistory(const EB_Position& pos, QBookEntry *be) 
{
  if (be==NULL) be=current_book;
  if (history_current<0 ||  
      history_book[history_current]!=be ||
      history[history_current].page!=pos.page ||
      history[history_current].offset!=pos.offset
      ) {
    history_current = history.size();
    history.resize(history.size() + 1);
    history_book.resize(history.size() + 1);
    history[history_current] = pos;
    history_book[history_current] = be;
  }
  if (history_current>0) emit makeHistory ();
}

void
Zten::goTo (EB_Position pos)
{
  if (!displaying) return;

  goToNoHistory (pos);

  /*
  if (history_current == (int) history.size() - 1)
    {
      history.resize(history.size() + 1);
      history_book.resize(history_book.size() + 1);
    }
  */
  addHistory(pos);
}

void
Zten::playSound (int index)
{
  current_book->playSound (index);
}

void
Zten::showImage (int index)
{
  current_book->showImage (index);
}

void
Zten::showImageGroup (int index)
{
  EB_Position pos;
  if (current_book->showImageGroup (index,pos)) {
    fprintf(stderr,"goto %d:%d\n", pos.page, pos.offset);
    goTo(pos);
  } else {
    fprintf(stderr,"goto =%d:%d\n", pos.page, pos.offset);
  }
}

void
Zten::playMovie (int index)
{
  current_book->playMovie (index);
}

void
Zten::goToNoHistory (EB_Position pos)
{
  
  setWaitCursor();
  QString s=current_book->getText (pos, ContentBrowser);

  if (s!="") {
    ContentBrowser->setMimeSourceFactory(QMimeSourceFactory::defaultFactory());
    ContentBrowser->setText (s);
  }
  resetWaitCursor();
}

void
Zten::goToHistory (int history_number)
{
  if (history_number>=0 && (unsigned int)history_number<history.size()) {
    current_book=history_book[history_number];
    goToNoHistory(history[history_number]);
  }
}

void
Zten::back ()
{
  emit followHistory ();
  if (history_current >= 1)
    {
      contentMode(MaximizeContent, false);
      goToHistory (--history_current);
    }
  if (history_current == 0)
    {
      emit beginningOfHistory ();
    }
}

void
Zten::forward ()
{
  emit followHistory ();
  if (history_current < (int) history.size() - 1)
    {
      contentMode(MaximizeContent, false);
      goToHistory(++history_current);
    }
  if (history_current == (int) history.size() - 1)
    {
      emit endOfHistory ();
    }
}

void
Zten::up ()
{
  if (history_current>=0) {
    current_book = history_book[history_current];
    EB_Position p = history[history_current];
    if (current_book->getPrevEntry(p)) goTo(p);
  }
}

void
Zten::down ()
{
  if (history_current>=0) {
    current_book = history_book[history_current];
    EB_Position p = history[history_current];
    if (current_book->getNextEntry(p)) goTo(p);
  }
}

void
Zten::gotoMenu (QBookEntry* be, EB_Position* p)
{
  if (p->page<=0) return;
  EntryList->clear();
  contentMode(MaximizeContent, false);
  current_book = be;
  goTo (*p);
}


void Zten::addMenuTitle(QBookEntry* be, int num, bool isImage) {
  QString title="";
  char buf[2];
  buf[1]=0;
  if (num<10) {
    buf[0]='0'+num;
    title=QString("(&")+buf+") ";
  } else if (num<36) {
    buf[0]='A'+num-10;
    title=QString("(&")+buf+") ";
  }
  title += be->getTitle();
  if (isImage) title+= " [image]";
  MenuSearch->insertItem (title,num,num);
}

void
Zten::findMenues (bool all)
{
  QBookEntry *be;
  MenuSearch->clear ();
  menu_books.clear ();
  menu_positions.clear();
  menu_positions.setAutoDelete(TRUE);

  int i=0;

  //  app->setOverrideCursor(WaitCursor);
  for (be = books.first (); be != NULL; be = books.next ()) {
    if (!all) {
      if (QBookEntry::group == DIC_GROUP_SINGLE) {
	if (be->priority[QBookEntry::group]<=0) continue;
      } else {
	if (be->priority[0]<=0 && be->priority[QBookEntry::group]<=0) continue;
	if (! be->menu_enabled) continue;
      }
    }
    EB_Position *p=new EB_Position;
    if (be->menuPosition(p)) {
      menu_books.append(be);
      menu_positions.append(p);
      addMenuTitle(be,i++,false);
    } else {
      delete p;
    }
    p=new EB_Position;
    if (be->imageMenuPosition(p)) {
      menu_books.append(be);
      menu_positions.append(p);
      addMenuTitle(be,i++,true);
    } else {
      delete p;
    }
  }
  //  app->restoreOverrideCursor();
  if (menu_books.count()==0) {
    MenuSearch->insertItem (tr("No Menu Search"),-1,-1);
  }
}

void
Zten::menuSelected (int index)
{
  if (menu_books.count () > 0)
    {
      MenuSearch->setEnabled (TRUE);
      QBookEntry *be = menu_books.at (index);
      gotoMenu (be, menu_positions.at(index));
    }
  else
    {
      MenuSearch->setEnabled (FALSE);
    }
}

void
Zten::findMultiSearches (bool all)
{
  QBookEntry *be;
  MultiSearch->clear ();
  ms_list.clear ();
  for (be = books.first (); be != NULL; be = books.next ()) {
    if (QBookEntry::group==DIC_GROUP_SINGLE) {
      if (be->priority[QBookEntry::group]<=0) continue;
    } else {
      if (!all && be->priority[0]<=0 && be->priority[QBookEntry::group]<=0) 
	continue;
    }
    be->findMultiSearch (all);
  }
  if (MultiSearch->count()==0) {
    MultiSearch->insertItem(tr("No Multi Search"));
  }
}

void Zten::updateMenuStatus() {
  MainWindow->updateMenuStatus();
}

void
Zten::multiSelected (int index)
{

  QBookEntry *be = NULL;
  QBookMultiSearch *ms = NULL;
  
  if (ms_list.count () > 0)
    {
      MultiSearch->setEnabled (TRUE);
      ms = ms_list.at (index);
      be = ms->getBookEntry ();
    }
  else
    {
      MultiSearch->setEnabled (FALSE);
    }
  if (be && ms && ms_list.count () > 0)
    {
      zm=new ZtenMultiSearch(this, ms_list.at(index));
      be->findMultiSearchEntry (ms->getMultiId ());
#ifdef ZAURUS
      zm->showMaximized ();
#else
      zm->show();
#endif
      zm->exec ();
      delete zm;
    }
}

void
Zten::addMultiSearch (QBookEntry *be, int multi_id, QString title, int)
{
  int num = MultiSearch->count();
  QString ttl="";
  if (num<10) {
    char buf[2];
    buf[0]='0'+num;
    buf[1]=0;
    ttl=QString("(&")+buf+") ";
  }
  MultiSearch->insertItem (ttl + be->getTitle () + " " + title, num, num);
  ms_list.append (new QBookMultiSearch (be, multi_id));	
}

void Zten::addMultiSearchEntry(QBookEntry *be,int a,QString b,bool c) {
  zm->addMultiSearchEntry (be,a,b,c);
};

int
Zten::openBooks (bool all, QProgressBar* progress)
{
  /*
    Return Value:
    -1: Ng
    0: Ok (all)
    1: Ok (core)
   */

  int flag = 0;
  //  app->setOverrideCursor(WaitCursor);
  DiskStat::checkDiskReady(config->EnableSD, config->EnableCF);

  //  QProgressBar *progress = NULL;
  if (config->EnableProgressBar) {
    //    progress=new QProgressBar(books.count(),this);
    //    progress->move(10,200);
    if (progress==NULL) 
      progress=new QProgressBar(books.count(),MainWindow);
    progress->move(0,0);
    progress->resize(200,30);
    progress->show();
  }
  setWaitCursor();

  //  app->processEvents();

  //  if (!opened) {
  int prog_count=0;
  for(QBookEntry *be=books.first(); be!=NULL; be=books.next()) {
    if (config->EnableProgressBar) {
      progress->setProgress(prog_count++);
    }
    if (all || be->priority[0]>0 || be->priority[QBookEntry::group]>0) {
      if (! be->isOpened() && be->isReady()) {
	if (!be->bindBook()) {
	  if (be->priority[0]>0) {
	    flag=-1;
	  } else if (flag==0) flag=1;
	}
      }
      if (be->isOpened() && be->isReady()) {
	disconnect(be, 
		   SIGNAL (foundMultiSearch(QBookEntry*,int,QString,int)),
		   0,0);
	disconnect(be,
		   SIGNAL (foundMultiSearchEntry(QBookEntry*,int,
						 QString,bool)),0,0);
	connect (be, SIGNAL (foundMultiSearch(QBookEntry*,int,
					      QString,int)),
		 this, SLOT (addMultiSearch (QBookEntry*,int,QString,int)));
	connect (be, SIGNAL (foundMultiSearchEntry(QBookEntry*,int,
						   QString,bool)),
		 this, SLOT (addMultiSearchEntry (QBookEntry*,int,
						  QString,bool)));
      }
    }
  }
  if (config->EnableProgressBar) {
    progress->setProgress(books.count());
    delete progress;
  }
  resetWaitCursor();
  return flag;
}

void 
Zten::closeSD() {
  for(QBookEntry *be=books.first(); be!=NULL; be=books.next()) {
    if (be->memory==SD_READY) {
      be->unbindBook();
    }
  }
}

void 
Zten::closeCF() {
  for(QBookEntry *be=books.first(); be!=NULL; be=books.next()) {
    if (be->memory==CF_READY) {
      be->unbindBook();
    }
  }
}

void
Zten::closeBooks ()
{
  //  opened = FALSE;
}

void
Zten::menuResume ()
{
}

void
Zten::reload ()
{
  //  ContentBrowser->setText ("");
  showBlank();
  current_book = NULL;

  
  /*
  openBooks ();
  findMenues ();
  findMultiSearches ();
  */
  DiskStat::disable();
  setFocus(FocusLineEdit);
}

void Zten::diskChanged() 
{
  fprintf(stderr,"DiskChanged\n");
  openBooks(false,NULL);
  findMenues();
  findMultiSearches();
}

void
Zten::paste()
{
  LineEdit->setEditText (QApplication::clipboard()->text());
  search();
}

void
Zten::copy()
{
  ContentBrowser->copy();
}

void
Zten::cut()
{
  copy();
  paste();
}

void
Zten::setPreferences ()
{
  GlobalConfig gconf (NULL, NULL, true);
#ifdef ZAURUS
  gconf.showMaximized ();
#else
  gconf.show ();
#endif
  if (gconf.exec () == QDialog::Accepted) {
    config->writeConfig(this);
    updateMenuStatus();
  }
}

void
Zten::selectFont ()
{
  ZtenFontMenu sf (config->FontFamily, config->FontSize, NULL, NULL, true);
#ifdef ZAURUS
  sf.showMaximized ();
#else
  sf.show ();
#endif
  if (sf.exec () == QDialog::Accepted) {
    QFont qf(sf.family());
    qf.setPixelSize(sf.size());
    setCurrentFont(qf);
  }
}

void Zten::dumpCurrent() {
  ZtenDump b (this, "Dump", true);

  if (history_current>=0) {
    current_book = history_book[history_current];
    EB_Position p = history[history_current];

    b.setAddress(current_book,p.page,p.offset);

  } else {
    b.setAddress(current_book,0,-1);
  }
#ifdef ZAURUS
  b.showMaximized ();
#else
  b.show ();
#endif
  b.exec();
}

extern "C" {
#if 0
  void zau_get_zio_status(char *buf, int fd);
#endif
};

void Zten::showInfo(){
  char buf[1024];
  //  ZtenDump b (this, "Info", true);
  //  b.TextView->setFont(QFont(config->FontFamily, config->FontSize));
  //  b.setAddress(tr("Ztenv Information"),0,-1);

  QString tmp, info;
  info=QString("Ztenv Version ")+QString(VERSION)+QString("\n");

  QString ppath="/proc/";
  tmp.setNum(getpid());
  ppath=QString("/proc/")+tmp+QString("/status");
  FILE* fp=fopen((const char *)ppath,"r");
  if (fp!=NULL) {
    while(fgets(buf,100,fp)!=NULL) {
      buf[100]=0;
      if (strncmp(buf,"VmSize:",7)==0 || strncmp(buf,"VmRSS:",6)==0) {
	info+=QString(buf);
      }
    }
    fclose(fp);
  }

#ifdef ZAURUS
#if 0
  zau_get_zio_status(buf, 0);
  info+=buf;
  info+="\n";
#endif
#endif
  tmp.setNum(books.count());
  info+=QString("Books = ")+tmp+QString("\n");
  int count=0;
  for(QBookEntry *be=books.first(); books.current(); be=books.next()) {
    ++count;
    sprintf(buf,"%03d: %c%c%c %s:%d\n", count,
	    be->isOpened()?'O':'-',
	    be->isReady()?'R':'-',
	    be->isBound()?'B':'-',
	    (const char *)(be->absolute_path),
	    be->sub_book_pos);
    info+=buf;
  }

  contentMode(MaximizeContent, false);
  ContentBrowser->setText(info);
  //  b.showMaximized ();
  //  b.exec();
}
