/*                                                            -*- C -*-
 * Copyright (c) 2004 Takashi Nemoto 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include <qobject.h>
#include <qstring.h>
#ifdef QTOPIA
#include <qpe/config.h>
#else
#include <qsettings.h>
class Config : public QSettings {
 public:
  Config(const QString& path) {
    setPath("ebsnap.lkj.jp",path, QSettings::User);
  }
  void setGroup(const QString& grp) {
    resetGroup();
    beginGroup(QString("ztenv/")+grp);
  };
  bool hasKey(const QString& keyname) {
    bool ok;
    readEntry(keyname,QString::null, &ok);
    return ok;
  };
};
#endif

#include "zten.h"
#include "ztenconf.h"
#include "qbookentry.h"

ZtenConf::~ZtenConf() {
}


ZtenConf::ZtenConf(const char *fname) {
  configFileName = fname;
  FontFamily = "lcfont";
  FontSize = 16;
  ImageScale = 100;
  IncrementalMode = true;
  AutoContentMode = false;
  AutoImageMode = true;
  BookNameBeforeEntry = true;
  MaxHits = 200;
  IncrementalMaxHits = 20;
  IncrementalMaxHitsEach = 5; 
  MultiMaxHits=100;
  KeywordMaxHits=100;
  ClipSearchMaxDicts=5;
  ClipSearchMaxHits=10;
  ClipSearchMaxHitsEach=5;
  GroupNumber=4;
  CurrentMatch=EXACT_MATCH;
  CursorKeyForScroll=true;
  Stemming = true;
  MovieCommand=QString("qcop QPE/Application/movieplayer \"setDocument2(QString,QString)\" \"%s\" \"\"");
  MaxContentLength=16000;

  EditToolbar=false;
  SearchToolbar=true;
  BrowseToolbar=false;
  StatusToolbar=false;
  UseSubSupFont=true;
  EnableScrollBar=true;
  EnableProgressBar=true;
  
  EnableQCopZtenv=true;
  EnableQCopZten=false;
  EnableQCopZdict=false;

  EnableCF=true;
  EnableSD=true;

  EnableClipSearch=false;

  EnableTitleBar=true;

  ImageSizeLimitMode = false;
  MaxImageWidth = 1280;
  MaxImageHeight = 1024;

  ResumeWait=1500;

  clipSearchMode=false;

  // key ztenv
  KeyConfigGroup=KEYCONF_ZTENV;
  keyconf[KEYCONF_ZTENV][KEYCONF_HIT  ]=KeyConfig(KEYCONF_UD,KEYCONF_NONE);
  keyconf[KEYCONF_ZTENV][KEYCONF_HIST ]=KeyConfig(KEYCONF_UD,KEYCONF_SHIFT);
  keyconf[KEYCONF_ZTENV][KEYCONF_ENTRY]=KeyConfig(KEYCONF_LR,KEYCONF_Fn);
  keyconf[KEYCONF_ZTENV][KEYCONF_SC1  ]=KeyConfig(KEYCONF_LR,KEYCONF_NONE);
  keyconf[KEYCONF_ZTENV][KEYCONF_SCP  ]=KeyConfig(KEYCONF_UD,KEYCONF_Fn);
  // key zdict
  keyconf[KEYCONF_ZDICT][KEYCONF_HIT  ]=KeyConfig(KEYCONF_UD,KEYCONF_NONE);
  keyconf[KEYCONF_ZDICT][KEYCONF_HIST ]=KeyConfig(KEYCONF_UD,KEYCONF_SHIFT);
  keyconf[KEYCONF_ZDICT][KEYCONF_ENTRY]=KeyConfig(KEYCONF_UD,KEYCONF_NONE);
  keyconf[KEYCONF_ZDICT][KEYCONF_SC1  ]=KeyConfig(KEYCONF_UD,KEYCONF_NONE);
  keyconf[KEYCONF_ZDICT][KEYCONF_SCP  ]=KeyConfig(KEYCONF_UD,KEYCONF_Fn);
  // key custom
  keyconf[KEYCONF_CUSTOM][KEYCONF_HIT  ]=KeyConfig(KEYCONF_UD,KEYCONF_NONE);
  keyconf[KEYCONF_CUSTOM][KEYCONF_HIST ]=KeyConfig(KEYCONF_UD,KEYCONF_SHIFT);
  keyconf[KEYCONF_CUSTOM][KEYCONF_ENTRY]=KeyConfig(KEYCONF_LR,KEYCONF_Fn);
  keyconf[KEYCONF_CUSTOM][KEYCONF_SC1  ]=KeyConfig(KEYCONF_LR,KEYCONF_NONE);
  keyconf[KEYCONF_CUSTOM][KEYCONF_SCP  ]=KeyConfig(KEYCONF_UD,KEYCONF_Fn);
}

void
ZtenConf::readConfig (Zten* zten)
{
  int n=0;
  Config* config=new Config(configFileName);
  config->setGroup ("Main");
  if (config->hasKey("FontFamily"))
    FontFamily = config->readEntry ("FontFamily");
  if (config->hasKey("FontSize"))
    FontSize = config->readNumEntry ("FontSize");
  if (FontSize<=0) FontSize=16;
  QFont qf(FontFamily);
  fprintf(stderr,"Size=%d\n",FontSize);
  qf.setPixelSize(FontSize);
  zten->setCurrentFont (qf);
  if (config->hasKey("ImageScale"))
    ImageScale = config->readNumEntry ("ImageScale");
  if (config->hasKey("IncrementalSearch"))
    IncrementalMode = config->readBoolEntry ("IncrementalSearch");
  if (config->hasKey("AutoContentDisplay"))
    AutoContentMode = config->readBoolEntry ("AutoContentDisplay");
  if (config->hasKey("AutoImageDisplay"))
    AutoImageMode = config->readBoolEntry ("AutoImageDisplay");

  if (config->hasKey("ImageSizeLimitMode"))
    ImageSizeLimitMode = config->readBoolEntry("ImageSizeLimitMode");
  if (config->hasKey("MaxImageWidth")) 
    MaxImageWidth = config->readNumEntry("MaxImageWidth");
  if (config->hasKey("MaxImageHeight"))
    MaxImageHeight = config->readNumEntry("MaxImageHeight");

  if (config->hasKey("Stemming"))
    Stemming = config->readBoolEntry ("Stemming");
  if (config->hasKey("MovieCommand"))
    MovieCommand = config->readEntry ("MovieCommand");
  if (config->hasKey("UseSubSupFont"))
    UseSubSupFont = config->readBoolEntry ("UseSubSupFont");

  if (config->hasKey("EnableQCopZtenv"))
    EnableQCopZtenv = config->readBoolEntry ("EnableQCopZtenv");
  if (config->hasKey("EnableQCopZten"))
    EnableQCopZten = config->readBoolEntry ("EnableQCopZten");
  if (config->hasKey("EnableQCopZdict"))
    EnableQCopZdict = config->readBoolEntry ("EnableQCopZdict");

  if (config->hasKey("EnableScrollBar"))
    EnableScrollBar = config->readBoolEntry ("EnableScrollBar");
  if (config->hasKey("EnableProgressBar"))
    EnableProgressBar = config->readBoolEntry ("EnableProgressBar");


  if (config->hasKey("EnableSD"))
    EnableSD = config->readBoolEntry ("EnableSD");
  if (config->hasKey("EnableCF"))
    EnableCF = config->readBoolEntry ("EnableCF");
  if (config->hasKey("ResumeWait"))
    ResumeWait = config->readNumEntry ("ResumeWait");

  if (config->hasKey("EnableClipSearch"))
    EnableClipSearch = config->readBoolEntry ("EnableClipSearch");

  if (config->hasKey("EnableTitleBar"))
    EnableTitleBar = config->readBoolEntry ("EnableTitleBar");
    

  if (config->hasKey("AltBookDisplay"))
    BookNameBeforeEntry = config->readBoolEntry ("AltBookDisplay");
  if (config->hasKey("MaxHits"))
    MaxHits =  config->readNumEntry ("MaxHits");
  if (config->hasKey("IncrementalSearchMaxHitsTotal"))
    IncrementalMaxHits =config->readNumEntry ("IncrementalSearchMaxHitsTotal");
  if (config->hasKey("IncrementalSearchMaxHitsEach"))
    IncrementalMaxHitsEach = 
      config->readNumEntry ("IncrementalSearchMaxHitsEach");
  if (config->hasKey("MultiMaxHits"))
    MultiMaxHits =  config->readNumEntry ("MultiMaxHits");
  if (config->hasKey("KeywordMaxHits"))
    KeywordMaxHits =  config->readNumEntry ("KeywordMaxHits");

  if (config->hasKey("ClipSearchMaxDicts"))
    ClipSearchMaxDicts =  config->readNumEntry ("ClipSearchMaxDicts");
  if (config->hasKey("ClipSearchMaxHits"))
    ClipSearchMaxHits =  config->readNumEntry ("ClipSearchMaxHits");
  if (config->hasKey("ClipSearchMaxHitsEach"))
    ClipSearchMaxHitsEach =  config->readNumEntry ("ClipSearchMaxHitsEach");

  if (config->hasKey("NumberOfBooks"))
    n = config->readNumEntry("NumberOfBooks");
  if (config->hasKey("GroupNumber"))
    QBookEntry::setGroupNumber(config->readNumEntry ("GroupNumber"));
  else 
    QBookEntry::setGroupNumber(GroupNumber);

  if (config->hasKey("MaxContentLength"))
    MaxContentLength=config->readNumEntry ("MaxContentLength");

  for(int i=0;i<MAX_DIC_GROUPS+2;i++) {
    QString keyname=QString("GroupName%1").arg(i);
    if (config->hasKey(keyname)) {
      GroupName[i]=config->readEntry(keyname);
    } else {
      switch(i) {
      case DIC_GROUP_ALL:
	GroupName[i]=tr("(&A) All Dictionaries");
	break;
      case DIC_GROUP_SINGLE:
	GroupName[i]=tr("(&S) Select one dictionary");
	break;
      default:
	GroupName[i]=tr("(&%1) Dictionary Group %2").arg(i).arg(i);
	break;
      }
    }
  }

  if (config->hasKey("EditToolbar"))
    EditToolbar=config->readBoolEntry ("EditToolbar");
  if (config->hasKey("SearchToolbar"))
    SearchToolbar=config->readBoolEntry ("SearchToolbar");
  if (config->hasKey("BrowseToolbar")) {
    BrowseToolbar=config->readBoolEntry ("BrowseToolbar");
  }
  if (config->hasKey("StatusToolbar"))
    StatusToolbar=config->readBoolEntry ("StatusToolbar");
  if (config->hasKey("CurrentMatch"))
    CurrentMatch=config->readNumEntry ("CurrentMatch");
  //  if (config->hasKey("CursorKeyForScroll"))
    //    CursorKeyForScroll=config->readBoolEntry("CursorKeyForScroll");
#ifndef QTOPIA
  config->endGroup();
#endif
  zten->books.clear();
  for (int i = 0; i < n; i++) {
    QString groupName;
    groupName.sprintf("Book%i", i);

    config->setGroup (groupName);

    QBookEntry *be = new QBookEntry(config->readEntry("BookTitle"),
				    config->readEntry("BookPath"),
				    config->readNumEntry("BookSubPos"));
    if (config->hasKey("AppendixPath")) 
      be->appendixPath=config->readEntry("AppendixPath");
    else
      be->appendixPath=QString("");

    if (config->hasKey("StopCodeMode")) {
      be->stop_code_mode = config->readNumEntry("StopCodeMode");
    } else {
      be->stop_code_mode = 0;

      if (config->hasKey("ContinuousMode") &&
	  config->readBoolEntry("ContinuousMode")) {
	be->stop_code_mode = 2;
      } else if (config->hasKey("AppendixEnabled") &&
		 config->readBoolEntry("AppendixEnabled")) {
	be->stop_code_mode = 1;
      }
    }

    if (config->hasKey("StopCode0") && config->hasKey("StopCode1")) {
      be->stop_code0=config->readNumEntry("StopCode0");
      be->stop_code1=config->readNumEntry("StopCode1");
    } else {
      be->stop_code0=be->stop_code1=0;
      if (be->stop_code_mode==0 && 
	  (be->absolute_path.find("/hdd3/dict1/genius")==0 ||
	   be->absolute_path.find("/hdd3/dict2/kojien")==0 ||
	   be->absolute_path.find("genius")>0)) {
	be->stop_code_mode=5;
	be->stop_code0=0x1f09;
	be->stop_code1=0x0001;
      }
    }

    for (int j=0;j<MAX_DIC_GROUPS+1;j++) {
      QString pname=QString("Priority%1").arg(j);
      if (config->hasKey(pname)) {
	be->priority[j]=config->readNumEntry(pname);
      } else {
	be->priority[j]=0;
      }
    }

    be->menu_enabled=config->readBoolEntry("MenuEnabled");
    //    be->image_menu_enabled=config->readBoolEntry("ImageMenuEnabled");

    be->n_multi_searches = config->readNumEntry("nMulti");
    for(int j=0;j<EB_MAX_MULTI_SEARCHES;j++) {
      be->multi_search_alt_title[j]="";
      be->multi_search_enabled[j]=false;
    }
    for(int j=0; j< be->n_multi_searches; j++) {
      QString s("Multi%1");
      be->multi_search_alt_title[j]=config->readEntry(s.arg(j));
      s="MultiEnable%1";
      be->multi_search_enabled[j]=config->readBoolEntry(s.arg(j));
    }


    be->setAltTitle(config->readEntry("BookAltTitle"), 
		    config->readEntry("BookAltTitleColor"));
    be->kanaConv=config->readBoolEntry("KanaConv");
    zten->books.append(be);
#ifndef QTOPIA
    config->endGroup();
#endif
  }
  zten->books.sort();
  delete config;
}

void
ZtenConf::writeConfig (Zten* zten)
{
  fprintf(stderr,"writeConfig Enter\n");
  Config* config=new Config(configFileName);

  config->setGroup ("Main");

  config->writeEntry ("FontFamily", FontFamily);
  config->writeEntry ("FontSize", FontSize);
  config->writeEntry ("ImageScale", ImageScale);
  config->writeEntry ("IncrementalSearch", IncrementalMode);
  config->writeEntry ("AutoContentDisplay", AutoContentMode);
  config->writeEntry ("AutoImageDisplay", AutoImageMode);
  config->writeEntry ("ImageSizeLimitMode", ImageSizeLimitMode);
  config->writeEntry ("MaxImageWidth", MaxImageWidth);
  config->writeEntry ("MaxImageHeight", MaxImageHeight);

  config->writeEntry ("MovieCommand", MovieCommand);
  config->writeEntry ("Stemming", Stemming);
  config->writeEntry ("UseSubSupFont", UseSubSupFont);

  config->writeEntry ("EnableQCopZtenv", EnableQCopZtenv);
  config->writeEntry ("EnableQCopZten", EnableQCopZten);
  config->writeEntry ("EnableQCopZdict", EnableQCopZdict);

  config->writeEntry ("EnableProgressBar", EnableProgressBar);
  config->writeEntry ("EnableScrollBar", EnableScrollBar);
  config->writeEntry ("EnableSD", EnableSD);
  config->writeEntry ("EnableCF", EnableCF);
  config->writeEntry ("ResumeWait", ResumeWait);

  config->writeEntry ("EnableClipSearch", EnableClipSearch);
  config->writeEntry ("EnableTitleBar", EnableTitleBar);
  config->writeEntry ("ClipSearchMaxDicts", ClipSearchMaxDicts);
  config->writeEntry ("ClipSearchMaxHits", ClipSearchMaxHits);
  config->writeEntry ("ClipSearchMaxHitsEach", ClipSearchMaxHitsEach);

  config->writeEntry ("AltBookDisplay", BookNameBeforeEntry);
  config->writeEntry ("MaxHits", MaxHits);
  config->writeEntry ("IncrementalSearchMaxHitsTotal", IncrementalMaxHits);
  config->writeEntry ("IncrementalSearchMaxHitsEach", IncrementalMaxHitsEach);
  config->writeEntry ("MultiMaxHits", MultiMaxHits);
  config->writeEntry ("KeywordMaxHits", KeywordMaxHits);
  config->writeEntry ("NumberOfBooks", int(zten->books.count()));
  config->writeEntry ("GroupNumber",   QBookEntry::getGroupNumber());
  for(int i=0;i<MAX_DIC_GROUPS+2;i++){
    QString keyname=QString("GroupName%1").arg(i);
    config->writeEntry(keyname, GroupName[i]);
  }
  config->writeEntry ("EditToolbar",   EditToolbar);
  config->writeEntry ("SearchToolbar", SearchToolbar);
  config->writeEntry ("BrowseToolbar", BrowseToolbar);
  config->writeEntry ("StatusToolbar", StatusToolbar);
  config->writeEntry ("CurrentMatch", CurrentMatch);
  //  config->writeEntry ("CursorKeyForScroll",CursorKeyForScroll);
  config->writeEntry ("MaxContentLength",MaxContentLength);
#ifndef QTOPIA
  config->endGroup();
#endif
  
  int number = 0;
  for (QBookEntry* be=zten->books.first();
       zten->books.current(); be=zten->books.next(), number++) {
    QString groupName;
    groupName.sprintf("Book%i", number);
    config->setGroup (groupName);
    config->writeEntry("BookPath",be->absolute_path);
    //    config->writeEntry("AppendixEnabled",be->appendix_enabled);
    config->writeEntry("AppendixPath",be->appendixPath);
    config->writeEntry("BookSubPos",be->sub_book_pos);
    config->writeEntry("BookTitle",be->getTitle());
    config->writeEntry("BookAltTitle",be->getAltTitle());
    config->writeEntry("BookAltTitleColor",be->getAltTitleColor());
    config->writeEntry("KanaConv",be->kanaConv);
    //    config->writeEntry("ContinuousMode", be->continuous_mode);
    config->writeEntry("StopCodeMode", be->stop_code_mode);
    config->writeEntry("StopCode0", be->stop_code0);
    config->writeEntry("StopCode1", be->stop_code1);

    for(int i=0;i<MAX_DIC_GROUPS+1;i++) {
      QString pname("Priority%1");
      config->writeEntry(pname.arg(i),be->priority[i]);
    }
    config->writeEntry("MenuEnabled",be->menu_enabled);
    // config->writeEntry("ImageMenuEnabled",be->image_menu_enabled);

    config->writeEntry("nMulti",be->n_multi_searches);
    for(int j=0; j< be->n_multi_searches; j++) {
      QString s("Multi%1");
      config->writeEntry(s.arg(j),be->multi_search_alt_title[j]);
      s="MultiEnable%1";
      config->writeEntry(s.arg(j),be->multi_search_enabled[j]);
    }
#ifndef QTOPIA
    config->endGroup();
#endif
  }

  delete config;
  fprintf(stderr,"writeConfig Exit\n");
}

