/*                                                            -*- C -*-
 * Copyright (c) 2003-2005 Teranishi & Nemoto 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include <dirent.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <qlistview.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qlineedit.h>
#include <qlabel.h>
#include "ztenext.h"
#include "qbookentry.h"
#include <eb/appendix.h>
#include "ztencolor.h"
#include "diskstat.h"

#ifndef QTOPIA
#include <qhbox.h>
#include <qlayout.h>
#include <qpushbutton.h>
#endif

ZtenExtension::ZtenExtension (QBookEntry *bep,
			      QWidget* parent, 
			      const char* name,
			      bool modal,
			      WFlags fl) : 
  ZtenExtensionBase(parent, name, modal, fl)
{
#ifndef ZAURUS
  QHBox* hbox=new QHBox(this);;
  QPushButton *acceptButton=new QPushButton(tr("Accept"),hbox);
  QPushButton *rejectButton=new QPushButton(tr("Reject"),hbox);
  hbox->setMaximumSize( QSize( 32000, 30 ) );
  connect((QObject *)acceptButton, SIGNAL (clicked()), this, SLOT (accept ()));
  connect((QObject *)rejectButton, SIGNAL (clicked()), this, SLOT (reject ()));
  ZtenExtensionBaseLayout->addWidget(hbox);
#endif  

  be = bep;

  if (!be->isOpened() && be->isReady()) {
    be->bindBook();
  } 

  const char *curd;
  int i = 0;
  int n_files, k, count = 0;
  EB_Appendix appendix;
  int set_pos = -1;

  HiraganaCheckBox->setChecked(be->kanaConv);
  TitleTextLabel->setText(be->title);
  PathTextLabel->setText(be->absolute_path);
  /*
  connect (AppendixCheckBox, SIGNAL (toggled (bool)), this,
	   SLOT (appendixToggled (bool)));  
  */
  connect (StopCodeComboBox, SIGNAL (activated (int)), this,
	   SLOT (stopcodeChanged ()));  
  connect ((QObject*)AltTitleColorTool, SIGNAL (clicked()), this,
	   SLOT(colorSelect()));
  for (curd = g_appendix_path[i].path;
	 curd != NULL; curd = g_appendix_path[++i].path) 
    {
      struct dirent **dir;
      if (! (chdir(curd) < 0))
	{
	  int pos = 0;
	  if ((n_files = scandir(".", &dir, 0, alphasort)) < 0) {
	    free(dir);
	    k=n_files;
	  } else {
	    for(k = 0; k < n_files; k++)
	      {
		if (strcmp(dir[k]->d_name, ".") == 0 || 
		    strcmp(dir[k]->d_name, "..") == 0) 
		  continue;
		eb_initialize_appendix (&appendix);
		if (eb_bind_appendix (&appendix, dir[k]->d_name) != EB_SUCCESS)
		  {
		    eb_finalize_appendix (&appendix);
		    free(dir[k]);
		    continue;
		  }
		QString set_appendix (dir[k]->d_name);
		if (g_appendix_path[i].name == NULL)
		  // 'directory_name/appendix_name'
		  {
		    set_appendix = curd + set_appendix;
		  }
		else
		  {
		    set_appendix = set_appendix + "(" +
		      g_appendix_path[i].name + ")";
		  }
		if (set_appendix == be->appendixPath) set_pos = pos;
		AppendixComboBox->insertItem (set_appendix);
		pos++;
		count++;
		eb_finalize_appendix (&appendix);
		free(dir[k]);
	      }
	  }
	  if (k == n_files && be->appendixPath!="") {
	    set_pos=AppendixComboBox->count();
	    AppendixComboBox->insertItem (be->appendixPath);
	  }
	}
      if (set_pos >= 0)
	{
	  AppendixComboBox->setCurrentItem (set_pos);
	}
  }
  chdir("/tmp");
  if (count == 0)
    {
      /*      AppendixCheckBox->setEnabled (FALSE); */
      AppendixComboBox->setEnabled (FALSE);
    }
  else if (be->appendixPath=="")
    {
      /*      AppendixCheckBox->setChecked(FALSE); */
      AppendixComboBox->setEnabled (FALSE);
    }
  
  if (be->n_multi_searches<1) EnableMultiSearch0->setEnabled(FALSE);
  if (be->n_multi_searches<2) EnableMultiSearch1->setEnabled(FALSE);
  if (be->n_multi_searches<3) EnableMultiSearch2->setEnabled(FALSE);
  if (be->n_multi_searches<4) EnableMultiSearch3->setEnabled(FALSE);
  if (be->n_multi_searches<5) EnableMultiSearch4->setEnabled(FALSE);
  if (be->n_multi_searches<6) EnableMultiSearch5->setEnabled(FALSE);

  /*  AppendixCheckBox->setChecked(be->appendix_enabled); */
  /*  ContinuousDisplay->setChecked(be->continuous_mode); */
  StopCodeComboBox->setCurrentItem(be->stop_code_mode);
  stopcodeChanged();

  if (be->alt_title == "") {
    AltTitleEntry->setText(be->title);
  } else {
    AltTitleEntry->setText(be->alt_title);
  }
  QString s;
  GroupPriority0->setText(s.sprintf("%3d",be->priority[0]));
  GroupPriority1->setText(s.sprintf("%3d",be->priority[1]));
  GroupPriority2->setText(s.sprintf("%3d",be->priority[2]));
  GroupPriority3->setText(s.sprintf("%3d",be->priority[3]));
  GroupPriority4->setText(s.sprintf("%3d",be->priority[4]));
  GroupPriority5->setText(s.sprintf("%3d",be->priority[5]));
  GroupPriority6->setText(s.sprintf("%3d",be->priority[6]));
  GroupPriority7->setText(s.sprintf("%3d",be->priority[7]));
  GroupPriority8->setText(s.sprintf("%3d",be->priority[8]));
  GroupPriority9->setText(s.sprintf("%3d",be->priority[9]));
  AltTitleColorEntry->setText(be->alt_title_color);
  //  IncWeightEntry->setText(inc_weight);

  for(int i=be->n_multi_searches; i< EB_MAX_MULTI_SEARCHES; i++) {
    be->multi_search_alt_title[i]="----";
  }

  
  if (be->multi_search_alt_title[0]=="")
    MultiSearchName0->setText(tr("multi search %1").arg(0));
  else
    MultiSearchName0->setText(be->multi_search_alt_title[0]);
  if (be->multi_search_alt_title[1]=="")
    MultiSearchName1->setText(tr("multi search %1").arg(1));
  else
    MultiSearchName1->setText(be->multi_search_alt_title[1]);
  if (be->multi_search_alt_title[2]=="")
    MultiSearchName2->setText(tr("multi search %1").arg(2));
  else
    MultiSearchName2->setText(be->multi_search_alt_title[2]);
  if (be->multi_search_alt_title[3]=="")
    MultiSearchName3->setText(tr("multi search %1").arg(3));
  else
    MultiSearchName3->setText(be->multi_search_alt_title[3]);
  if (be->multi_search_alt_title[4]=="")
    MultiSearchName4->setText(tr("multi search %1").arg(4));
  else
    MultiSearchName4->setText(be->multi_search_alt_title[4]);
  if (be->multi_search_alt_title[5]=="")
    MultiSearchName5->setText(tr("multi search %1").arg(5));
  else
    MultiSearchName5->setText(be->multi_search_alt_title[5]);
  
  EnableMultiSearch0->setChecked(be->multi_search_enabled[0]);
  EnableMultiSearch1->setChecked(be->multi_search_enabled[1]);
  EnableMultiSearch2->setChecked(be->multi_search_enabled[2]);
  EnableMultiSearch3->setChecked(be->multi_search_enabled[3]);
  EnableMultiSearch4->setChecked(be->multi_search_enabled[4]);
  EnableMultiSearch5->setChecked(be->multi_search_enabled[5]);

  EB_Position p;
  if (be->menuPosition(&p) || be->imageMenuPosition(&p)) {
    EnableMenuSearch->setChecked(be->menu_enabled);
  } else {
    EnableMenuSearch->setEnabled(FALSE);
    EnableMenuSearch->setChecked(FALSE);
  }

}

ZtenExtension::~ZtenExtension ()
{
}

/*
void ZtenExtension::appendixToggled (bool on)
{
  if (on)
    {
    AppendixComboBox->setEnabled (TRUE);    
    }
  else
    {
      AppendixComboBox->setEnabled (FALSE);
    }
}
*/

void ZtenExtension::stopcodeChanged()
{
  if (StopCodeComboBox->currentItem()==1) {
    AppendixComboBox->setEnabled (TRUE);
  } else {
    AppendixComboBox->setEnabled (FALSE);
  }
}


void ZtenExtension::colorSelect(void) {
  
  ZtenColor zc(AltTitleColorEntry->text().mid(1).toInt(NULL,16), 
	       this, NULL, true);
  zc.showMaximized();
  if (zc.exec()) {
    char buf[10];
    sprintf(buf,"#%06X",zc.color());
    AltTitleColorEntry->setText(buf);
  }
}

void ZtenExtension::accept ()
{
  be->kanaConv=HiraganaCheckBox->isChecked();
  be->stop_code_mode = StopCodeComboBox->currentItem();
  if (StopCodeComboBox->currentText()[0]==' ') {
    // ==== Hard Coded ====
    be->stop_code0=StopCodeComboBox->currentText().mid(1,4).toInt(0,16);
    be->stop_code1=StopCodeComboBox->currentText().mid(6,4).toInt(0,16);
  }
  /*
  be->appendix_enabled=AppendixCheckBox->isChecked ();
  */
  if (be->stop_code_mode==1 && AppendixComboBox->currentText () != "") {
    be->setAppendix(AppendixComboBox->currentText ());
  }

  be->setAltTitle(AltTitleEntry->text(), AltTitleColorEntry->text());

  be->priority[0]=GroupPriority0->text().toInt();
  be->priority[1]=GroupPriority1->text().toInt();
  be->priority[2]=GroupPriority2->text().toInt();
  be->priority[3]=GroupPriority3->text().toInt();
  be->priority[4]=GroupPriority4->text().toInt();
  be->priority[5]=GroupPriority5->text().toInt();
  be->priority[6]=GroupPriority6->text().toInt();
  be->priority[7]=GroupPriority7->text().toInt();
  be->priority[8]=GroupPriority8->text().toInt();
  be->priority[9]=GroupPriority9->text().toInt();
  for(int i=0;i<MAX_DIC_GROUPS+1;i++) if (be->priority[i] < 0) be->priority[i]=0;

  be->menu_enabled=EnableMenuSearch->isChecked();
  //  be->continuous_mode=ContinuousDisplay->isChecked();

  if (be->n_multi_searches > 0) {
    be->multi_search_alt_title[0]=MultiSearchName0->text();
    be->multi_search_enabled[0]=EnableMultiSearch0->isChecked();
  }
  if (be->n_multi_searches > 1) {
    be->multi_search_alt_title[1]=MultiSearchName1->text();
    be->multi_search_enabled[1]=EnableMultiSearch1->isChecked();
  }
  if (be->n_multi_searches > 2) {
    be->multi_search_alt_title[2]=MultiSearchName2->text();
    be->multi_search_enabled[2]=EnableMultiSearch2->isChecked();
  }
  if (be->n_multi_searches > 3) {
    be->multi_search_alt_title[3]=MultiSearchName3->text();
    be->multi_search_enabled[3]=EnableMultiSearch3->isChecked();
  }
  if (be->n_multi_searches > 4) {
    be->multi_search_alt_title[4]=MultiSearchName4->text();
    be->multi_search_enabled[4]=EnableMultiSearch4->isChecked();
  }
  if (be->n_multi_searches > 5) {
    be->multi_search_alt_title[5]=MultiSearchName5->text();
    be->multi_search_enabled[5]=EnableMultiSearch5->isChecked();
  }

  ZtenExtensionBase::accept();
}
