#include <qlabel.h>
#include <qstring.h>
#include <qobject.h>
#include <qtextcodec.h>
#include "ztenfontmenu.h"
#include <qfontdatabase.h>
#include <qcombobox.h>
#include <cstdio>
#include <cstdlib>

#ifndef QTOPIA
#include <qhbox.h>
#include <qlayout.h>
#include <qpushbutton.h>
#endif

ZtenFontMenu::ZtenFontMenu(QString family, int size, QWidget* parent, 
			   const char * name, bool modal, WFlags fl):
  ZtenFontMenuBase(parent,name,modal,fl)
{
#ifndef QTOPIA
  QHBox* hbox=new QHBox(this);;
  QPushButton *acceptButton=new QPushButton(tr("Accept"),hbox);
  QPushButton *rejectButton=new QPushButton(tr("Reject"),hbox);
  hbox->setMaximumSize( QSize( 32000, 30 ) );
  connect((QObject *)acceptButton, SIGNAL (clicked()), this, SLOT (accept ()));
  connect((QObject *)rejectButton, SIGNAL (clicked()), this, SLOT (reject ()));
  ZtenFontMenuBaseLayout->addWidget(hbox);
#endif  

  SampleNormal->setText(tr("Normal: ABCDabcd"));
  SampleBold->setText(tr("Bold: ABCDabcd"));
  SampleItalic->setText(tr("Italic: ABCDabcd"));
  SampleBoldItalic->setText(tr("BoldItalic: ABCDabcd"));

  setSampleFont(family, size);

  QFontDatabase qfd;
  QStringList fontFamilies=qfd.families(TRUE);
  for(QStringList::Iterator it=fontFamilies.begin(); 
      it!=fontFamilies.end();++it) {
    if (*it != QString("ebsup") && *it != QString("ebsub")) {
      FontFamilyBox->insertItem(*it);
    }
  }  

  connect(FontFamilyBox, SIGNAL(activated(const QString &)),this, SLOT(familyChanged()));
  connect(SizeBox, SIGNAL(activated(const QString &)),this, SLOT(sizeChanged()));
  setCurrent(family, size);
  setSampleFont(family, size);
}
 

bool ZtenFontMenu::setSampleFont(QString& family, int& size) {
  QFont qf(family);
  qf.setPixelSize(size);
  fprintf(stderr,"Size=%d\n",size);
  qf.setBold(FALSE); qf.setItalic(FALSE); 
  SampleNormal->setFont(QFont(family, size, QFont::Normal, FALSE));
  qf.setBold(TRUE);  qf.setItalic(FALSE); 
  SampleBold->setFont(QFont(family, size, QFont::Bold, FALSE));
  qf.setBold(FALSE); qf.setItalic(TRUE); 
  SampleItalic->setFont(QFont(family, size, QFont::Normal, TRUE));
  qf.setBold(TRUE);  qf.setItalic(TRUE); 
  SampleBoldItalic->setFont(QFont(family, size, QFont::Bold, TRUE));
  
  QFontInfo fi(SampleNormal->font());
  bool f=true;
  if (family!=fi.family()) {
    family=fi.family();
    f=false;
  }
#ifdef QTOPIA
  if (size!=fi.pointSize()) {
    size=fi.pointSize();
    f=false;
  }
#else
  if (size!=fi.pixelSize()) {
    size=fi.pixelSize();
    f=false;
  }
#endif
  return f;
}

ZtenFontMenu::~ZtenFontMenu() 
{
}

void ZtenFontMenu::familyChanged() {
  QFontDatabase qfd;
  int df,nsz;
  QString family=FontFamilyBox->currentText();
  int sz=nsz=SizeBox->currentText().toInt();
  df=9999;
  SizeBox->clear();
  QValueList<int> ps=qfd.pointSizes(family);
#ifndef QTOPIA
  QFont qf(family);
#endif
  //  int point=0;
  for(QValueList<int>::Iterator it2=ps.begin(); it2!=ps.end(); ++it2) {
#ifdef QTOPIA
    int cs=(*it2)/10;
#else
    qf.setPointSize(*it2);
    int cs=QFontInfo(qf).pixelSize();
#endif
    SizeBox->insertItem(QString().setNum(cs));
    if (abs(sz-cs)<df) {
      df=abs(sz-cs);
      nsz=cs;
    }
  }
  int c=SizeBox->count();
  for(int i=0;i<c; i++){
    if (SizeBox->text(i).toInt() == nsz) {
      SizeBox->setCurrentItem(i);
      break;
    }
  }
  setSampleFont(family,nsz);
}

void ZtenFontMenu::sizeChanged() {
  QString family=FontFamilyBox->currentText();
  int sz=SizeBox->currentText().toInt();
  setSampleFont(family,sz);
}

void ZtenFontMenu::setCurrent(const QString& family, int size) {
  int c=FontFamilyBox->count();
  for(int i=0;i<c; i++){
    if (FontFamilyBox->text(i) == family) {
      FontFamilyBox->setCurrentItem(i);
      break;
    }
  }
  familyChanged();
  c=SizeBox->count();
  for(int i=0;i<c; i++){
    if (SizeBox->text(i).toInt() == size) {
      SizeBox->setCurrentItem(i);
      break;
    }
  }
  sizeChanged();
}

QString ZtenFontMenu::family() {
  return FontFamilyBox->currentText();
}

int ZtenFontMenu::size() {
  return SizeBox->currentText().toInt();
}
