#ifndef ZTENWINDOW_H
#define ZTENWINDOW_H

#include <qmainwindow.h>
#include "zten.h"

class QAction;
class QActionGroup;
class QPixmap;
class QToolBar;
class QHistoryLineEdit;
class QLineEdit;
class MyPushButton;
class QPEMenuBar;
class QMenuBar;

class ZtenWindow : public QMainWindow
{
  friend class Zten;

  Q_OBJECT
public:
  ZtenWindow (QWidget *parent = 0);
  void resume ();
  void checkIncremental(bool on);
  void checkAutoContent(bool on);
  void checkClipSearch(bool on);
//  void checkTitleBar(bool on);
  void openTrig();
  void setupDictionaryGroupMenu();

protected:
  MyPushButton* DelButton;
  QHistoryLineEdit* LineEdit;
  //  MyPushButton* LookupButton;
  QLineEdit* JumpEdit;
  void updateMenuStatus();
  virtual void resizeEvent(QResizeEvent *);
  //  virtual void showMaximized();

private slots:
  /*
  void selectBooks ();
  void deleteBook ();  
  */
  void copy ();
  void paste ();
  void back ();
  void forward ();
  void down ();
  //void gotoMenu ();
  /*
  void listUp ();
  void listDown ();
  void setExtension ();
  */

  void closeFiles();
  
  void enableNavigations ();     // enable back, forward
  void enableBackNavigations (); // enable back, disable forward
  void disableNavigations (); // disable back, forward
  void disableBack ();	       // disable back
  void disableForward ();     // disable forward

  void setIncremental (bool);
  void toggleIncremental ();
  void setAutoContent (bool);
  void toggleAutoContent ();

  void setClipSearch (bool);
  void toggleClipSearch ();

  void setPreferences ();

  void setEditTool (bool);
  void setBrowseTool (bool);
  void setSearchTool (bool);
//  void setTitleBar (bool);

  void toggleEditTool ();
  void toggleBrowseTool ();
  void toggleSearchTool ();
//  void toggleTitleBar ();


  //  void setCurrentFont (int);
  // void setCurrentFont (void);
  void setImageScale (int);
  void setCurrentMatch (int);

  void dispatchMessage (const QCString &msg, const QByteArray &data);
//  void checkDiskReady ();
//  void checkDiskReady (bool forceUpdate);
  void drawToolBar ();

  void fontList();

  void dicList();

  /*
  void setDictionaryGroup0 ();
  void setDictionaryGroup1 ();
  void setDictionaryGroup2 ();
  void setDictionaryGroup3 ();
  void setDictionaryGroup4 ();
  */
  void setDictionaryGroupId (int n);
  void selectSingleDictionary(int n);

  void dumpCurrent();
  void showInfo();
 signals:
  void reloadBooks ();

public slots:
  void lookup1(QString);    
  void lookup1();    

  void exitApplication();
#ifdef ZAURUS
  void checkClipboard();
#endif

private:
  void lookup (QString);
  void lookup ();

  void setDictionaryGroup (int n);

  void checkDictionaryGroup(int);
  void checkCurrentMatch(int);
  void checkImageScale(int);


  QPopupMenu *makeFontMenu();
 
  //  QPixmap smallPixmap(const char* xpm[]);
  Zten *zten;

  QToolBar *menuBarBase;
#ifdef ZAURUS
  QPEMenuBar *menuBar;
#else
  QMenuBar *menuBar;
#endif
  QPopupMenu *menu;
  QPopupMenu *font;
  QPopupMenu *match;
  QPopupMenu *scale; 
  QPopupMenu *menuConfigToolBar, *menuDictionaryGroup;

  QAction *listDownAction, *copyAction, *pasteAction,
    *backAction, *forwardAction, *extensionAction, *downAction,
    *listUpAction, *selectBooksAction, *deleteBookAction;

  //  QAction *cfStatus, *sdStatus;
  //  QActionGroup *browseBox, *editBox, *statusBox;
//  QActionGroup *browseBox, *editBox;
  QToolBar *toolBar, *searchBar;
  QToolBar *browseBar, *editBar;

  QTimer *timer, *timer2,*timer3;
  QString lastClipboard;
  int clipSearchCount;

  int timer_counter;

  //*homeAction;
  int incremental_id;
  int autoContent_id;
  int clipSearch_id;
  int editTool_id;
//  int titleBar_id;
  //  int statusTool_id;
  int browseTool_id;
  int dicSortTool_id;
  int* dicGroup_id;
  int searchTool_id;
};

#endif // ZTENWINDOW_H
